<?php
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/includes/database.php $
 * $Id: database.php 1360 2009-01-01 21:27:41Z Quietust $
 */

if (!defined('IN_GAME') && !defined('IN_TURNS'))
	die("Access denied");

// Supported database types:
// 'mysql' - MySQL 4.1 or greater

function db_open ($type, $sock, $host, $port, $user, $pass, $name)
{
	if (!strlen($sock) && !strlen($host))
		die('Invalid database configuration - must specify either hostname or UNIX socket');
	if (strlen($sock) && strlen($host))
		die('Invalid database configuration - cannot specify both hostname and UNIX socket');

	// Select database driver and construct DSN for connection
	switch ($type)
	{
	case 'mysql':
		$dsn = 'mysql:';
		if ($sock)
			$dsn .= 'unix_socket='. $sock .';';
		elseif ($host)
		{
			$dsn .= 'host='. $host .';';
			if ($port)
				$dsn .= 'port='. $port .';';
		}
		$dsn .= 'dbname='. $name;
		break;
	default:
		die('An unsupported database driver has been specified!');
	}

	// Attempt to open database connection
	try
	{
		$db = new PDO($dsn, $user, $pass);
	}
	catch (PDOException $e)
	{
		return NULL;
	}

	$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);

	// Database-specific post-connect actions
	switch ($type)
	{
	case 'mysql':
		$db->query("SET NAMES 'utf8'");
		$db->query("SET SESSION sql_mode=''");
		break;
	}

	return $db;
}

// Shortcut functions for binding Int versus String
function db_bindIntValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_INT);
}
function db_bindStrValue ($query, $parm, $value)
{
	$query->bindValue($parm, $value, PDO::PARAM_STR);
}

// evaluate an SQL query, return first cell of first row
// useful for "SELECT COUNT(*) ..." queries
function sqleval ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed!', 1);
	$q->execute($parms) or warning('SQL execute failed!', 1);
	return $q->fetchColumn();
}

// 1-liner for executing quick SQL queries
function sqlquery ($db, $query, $parms = NULL)
{
	$q = $db->prepare($query) or warning('SQL prepare failed!', 1);
	$q->execute($parms) or warning('SQL execute failed!', 1);
	return $q;
}
?>
