<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/includes/logging.php $
 * $Id: logging.php 1360 2009-01-01 21:27:41Z Quietust $
 */

if (!defined('IN_GAME') && !defined('IN_TURNS'))
	die("Access denied");

function error_handler ($errno, $errstr, $errfile, $errline)
{
	// Do NOT attempt to log any errors that occur within this file
	// or it will likely throw itself into an infinite loop
	if ($errfile == __FILE__)
		return FALSE;
	logmsg($errno, "$errstr in '$errfile' on line '$errline'");
	return FALSE;
}

// Generates a warning message
// If in the setup script, display as formatted XHTML
// If in turns script, display as plain text (to be logged to disk)
// If in-game, store in database (along with other relevant information)
// Specify level = 0 to indicate the file/line where warning() itself was called,
// level = 1 for the caller, level = 2 for the caller's caller, etc.
function warning ($text, $level)
{
	$dbg = debug_backtrace();
	if (!isset($dbg[$level]))
		$level = 0;
	$file = $dbg[$level]['file'];
	$line = $dbg[$level]['line'];
	if (defined('IN_SETUP'))
		echo "<br /><b>Warning</b>:  $text in <b>$file</b> on line <b>$line</b><br />\n";
	elseif (defined('IN_GAME'))
		logmsg(E_USER_WARNING, "$text in '$file' on line '$line'");
	elseif (defined('IN_TURNS'))
		echo "\nWarning:  '$text' in $file on line $line\n\n";
}

// If logging is enabled, logs the current event
function logevent ($text = '')
{
	if (!LOG_ENABLE)
		return;
	logmsg(0, $text);
}

// Log an event into the database
function logmsg ($type, $text = '')
{
	global $page, $action, $lock;
	global $db;
	$q = $db->prepare('INSERT INTO '. LOG_TABLE .' (log_time,log_type,log_ip,log_page,log_action,log_locks,log_text) VALUES (?,?,?,?,?,?,?)');
	db_bindIntValue($q, 1, CUR_TIME);
	db_bindIntValue($q, 2, $type);
	db_bindIntValue($q, 3, $_SERVER['REMOTE_ADDR']);
	db_bindStrValue($q, 4, $page);
	db_bindStrValue($q, 5, $action);

	$lockstr = '';
	if (isset($lock))
	{
		foreach ($lock as $name => $val)
			if ($val > 0)
				$lockstr .= ','. substr($name, 0, 1) . $val;
		$lockstr = trim($lockstr, ',');
	}
	if (!strlen($lockstr))
		$lockstr = 'n/a';

	db_bindStrValue($q, 6, $lockstr);
	db_bindStrValue($q, 7, $text);
	$q->execute();	// can't display a warning message here, or it'd recurse onto itself
}

// When passing get_defined_vars() as second parameter, fetches all variables whose names are in the first parameter
// and formats them into a text string suitable for inserting into the log database table
function varlist ($names, $vars)
{
	$arr = array();
	foreach ($names as $name)
		$arr[$name] = $vars[$name];		
	$data = explode("\n", var_export($arr, TRUE));
	array_pop($data);
	array_shift($data);
	$ret = implode('', $data);
	return trim(str_replace(array('array ( ', ', )', ' => '), array('arr[', ']', ':'), preg_replace('/ +/', ' ', $ret)));
}
?>
