<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/install/setup.php $
 * $Id: setup.php 1360 2009-01-01 21:27:41Z Quietust $
 */

error_reporting(E_ALL | E_STRICT);
date_default_timezone_set('UTC');
ignore_user_abort(1);
set_time_limit(0);

if (!isset($_SERVER['REQUEST_URI']) || defined('IN_TURNS'))
	die("Access denied");

define('IN_GAME', TRUE);
define('IN_SETUP', TRUE);
define('PROM_BASEDIR', dirname(__FILE__) . '/');

// Make sure setup.php is running from the base folder
if (!file_exists(PROM_BASEDIR .'config.php') || !file_exists(PROM_BASEDIR .'index.php') || !file_exists(PROM_BASEDIR .'turns.php'))
	die("Access denied - this game has already been configured.");

require(PROM_BASEDIR .'config.php');
require(PROM_BASEDIR .'includes/constants.php');
require(PROM_BASEDIR .'includes/language.php');
require(PROM_BASEDIR .'includes/html.php');
require(PROM_BASEDIR .'includes/logging.php');
require(PROM_BASEDIR .'includes/misc.php');
require(PROM_BASEDIR .'includes/database.php');
require(PROM_BASEDIR .'classes/prom_vars.php');
require(PROM_BASEDIR .'classes/prom_user.php');
require(PROM_BASEDIR .'classes/prom_empire.php');

$db = db_open(DB_TYPE, DB_SOCK, DB_HOST, DB_PORT, DB_USER, DB_PASS, DB_NAME);
if (!$db)
	error_503('ERROR_TITLE', 'Unable to connect to database');

$action = getFormVar('action');

$html = new prom_html_compact();
$html->begin('Setup');

$username = getFormVar('admin_username');
$password = getFormVar('admin_password');
$password_verify = getFormVar('admin_password_verify');
$name = htmlspecialchars(getFormVar('admin_name'));
$email = htmlspecialchars(getFormVar('admin_email'));
$empname = htmlspecialchars(getFormVar('admin_empirename'));

// by default, start in 24-48 hours
$deftime = CUR_TIME - (CUR_TIME % 86400) + 86400 * 2;

$begin = getFormVar('round_begin', gmdate('Y/m/d H:i:s O', $deftime));
$closing = getFormVar('round_closing', gmdate('Y/m/d H:i:s O', $deftime + 86400 * 21));
$end = getFormVar('round_end', gmdate('Y/m/d H:i:s O', $deftime + 86400 * 28 - 1));

if ($action == 'submit') do
{
	if (empty($username))
	{
		notice(lang('INPUT_NEED_USERNAME'));
		break;
	}
	if (empty($password))
	{
		notice(lang('INPUT_NEED_PASSWORD'));
		break;
	}
	if (strlen($username) > 255)
	{
		notice(lang('INPUT_USERNAME_TOO_LONG'));
		break;
	}
	if (empty($name))
	{
		notice(lang('INPUT_NEED_NICKNAME'));
		break;
	}
	if (strlen($name) > 255)
	{
		notice(lang('INPUT_NICKNAME_TOO_LONG'));
		break;
	}
	if (!validate_email($email))
	{
		notice(lang('INPUT_NEED_EMAIL'));
		break;
	}
	if (strlen($email) > 255)
	{
		notice(lang('INPUT_EMAIL_TOO_LONG'));
		break;
	}
	if ($password != $password_verify)
	{
		notice(lang('INPUT_PASSWORD_MISMATCH'));
		break;
	}
	if (empty($empname))
	{
		notice(lang('INPUT_NEED_EMPIRE'));
		break;
	}
	if (strlen($empname) > 255)
	{
		notice(lang('INPUT_EMPIRE_TOO_LONG'));
		break;
	}

	$_begin = strtotime($begin);
	$_closing = strtotime($closing);
	$_end = strtotime($end);

	if ($_begin == FALSE)
	{
		notice(lang('SETUP_BAD_START'));
		break;
	}
	if ($_closing == FALSE)
	{
		notice(lang('SETUP_BAD_COOLDOWN'));
		break;
	}
	if ($_end == FALSE)
	{
		notice(lang('SETUP_BAD_END'));
		break;
	}

	if ($_begin < CUR_TIME)
	{
		notice(lang('SETUP_START_IN_FUTURE'));
		break;
	}

	if ($_begin >= $_closing)
	{
		notice(lang('SETUP_COOLDOWN_AFTER_START'));
		break;
	}

	if ($_closing + (VACATION_START + VACATION_LIMIT) * 3600 >= $_end)
	{
		notice(lang('SETUP_COOLDOWN_LENGTH', ceil((VACATION_START + VACATION_LIMIT) / 24)));
		break;
	}

	// empty out all database tables
	if (!$db->query('TRUNCATE TABLE '. CLAN_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', CLAN_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. CLAN_RELATION_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', CLAN_RELATION_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. CLAN_TOPIC_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', CLAN_TOPIC_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. CLAN_MESSAGE_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', CLAN_MESSAGE_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. CLAN_NEWS_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', CLAN_NEWS_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. EMPIRE_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', EMPIRE_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. EMPIRE_EFFECT_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', EMPIRE_EFFECT_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. EMPIRE_NEWS_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', EMPIRE_NEWS_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. EMPIRE_MESSAGE_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', EMPIRE_MESSAGE_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. LOG_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', LOG_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. LOCK_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', LOCK_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. LOTTERY_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', LOTTERY_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. MARKET_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', MARKET_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. PERMISSION_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', PERMISSION_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. USER_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', USER_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. VAR_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', VAR_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. VAR_ADJUST_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', VAR_ADJUST_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. HISTORY_CLAN_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', HISTORY_CLAN_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. HISTORY_EMPIRE_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', HISTORY_EMPIRE_TABLE));
		break;
	}
	if (!$db->query('TRUNCATE TABLE '. HISTORY_ROUND_TABLE))
	{
		notice(lang('SETUP_ERROR_CLEAR_TABLE', HISTORY_ROUND_TABLE));
		break;
	}

	// lock world variables - if this fails, something has gone horribly wrong
	$world = new prom_vars();
	if (!$world->lock(1))
	{
		notice(lang('SETUP_ERROR_LOCK_WORLD'));
		break;
	}

	// create world variables
	$world->create('lotto_current_jackpot');
	$world->create('lotto_yesterday_jackpot');
	$world->create('lotto_last_picked');
	$world->create('lotto_last_winner');
	$world->create('lotto_jackpot_increase');
	$world->create('round_time_begin');
	$world->create('round_time_closing');
	$world->create('round_time_end');

	$world->lotto_current_jackpot = LOTTERY_JACKPOT;
	$world->lotto_yesterday_jackpot = LOTTERY_JACKPOT;
	$world->lotto_last_picked = 0;
	$world->lotto_last_winner = 0;
	$world->lotto_jackpot_increase = 0;

	$world->round_time_begin = $_begin;
	$world->round_time_closing = $_closing;
	$world->round_time_end = $_end;

	// create user account
	$user = new prom_user();
	$user->create($username, $password, $name, $email);
	$user->u_flags = UFLAG_ADMIN | UFLAG_MOD;
	$user->u_lastip = $_SERVER['REMOTE_ADDR'];

	$emp = new prom_empire();
	$emp->create($user, $empname, 1);
	$emp->e_flags = EFLAG_ADMIN | EFLAG_VALID;

	$user->save();
	$emp->save();
	$world->save();

	notice(lang('SETUP_COMPLETE'));
	notices();
	$html->end();
	exit;
} while (0);
?>
<h2><?=lang('SETUP_TITLE')?></h2>
<p><?=lang('SETUP_DESC')?></p>
<?
notices(2);
?>
<form method="post" action="setup.php">
<table class="inputtable">
<tr><th colspan="2"><?=lang('SETUP_ADMIN')?></th></tr>
<tr><th class="ar"><?=lang('LABEL_USERNAME')?></th>
    <td><input type="text" name="admin_username" value="<?=htmlspecialchars($username)?>" size="8" maxlength="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_PASSWORD')?></th>
    <td><input type="password" name="admin_password" size="8" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_PASSWORD_VERIFY')?></th>
    <td><input type="password" name="admin_password_verify" size="8" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_NICKNAME')?></th>
    <td><input type="text" name="admin_name" value="<?=$name?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_EMAIL')?></th>
    <td><input type="text" name="admin_email" value="<?=$email?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_EMPIRE')?></th>
    <td><input type="text" name="admin_empirename" value="<?=$empname?>" size="24" maxlength="32" /></td></tr>
<tr><th colspan="2"><hr /></th></tr>
<tr><th colspan="2"><?=lang('SETUP_SETTINGS')?></th></tr>
<tr><th class="ar"><?=lang('LABEL_ROUND_START')?></th>
    <td><input type="text" name="round_begin" value="<?=$begin?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_ROUND_COOLDOWN')?></th>
    <td><input type="text" name="round_closing" value="<?=$closing?>" size="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_ROUND_END')?></th>
    <td><input type="text" name="round_end" value="<?=$end?>" size="24" /></td></tr>
<tr><th colspan="2"><hr /></th></tr>
<tr><td colspan="2" class="ac"><input type="hidden" name="action" value="submit" /><input type="submit" value="<?=lang('SETUP_SUBMIT')?>" /></td></tr>
</table>
</form>
<?
$html->end();
?>
