<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/pages/pubmarketsell.php $
 * $Id: pubmarketsell.php 1360 2009-01-01 21:27:41Z Quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

$title = 'PUBMARKETSELL_TITLE';

require(PROM_BASEDIR .'includes/header.php');

if (ROUND_FINISHED)
	unavailable(lang('PUBMARKETSELL_UNAVAILABLE_END'));
if (!ROUND_STARTED)
	unavailable(lang('PUBMARKETSELL_UNAVAILABLE_START'));
if ($emp1->is_protected())
	unavailable(lang('PUBMARKETSELL_UNAVAILABLE_PROTECT'));
if ($emp1->e_flags & EFLAG_ADMIN)
	unavailable(lang('PUBMARKETSELL_UNAVAILABLE_ADMIN'));

$types = array(MARKET_TRPARM => 'trparm', MARKET_TRPLND => 'trplnd', MARKET_TRPFLY => 'trpfly', MARKET_TRPSEA => 'trpsea', MARKET_FOOD => 'food');
$stores = array('trparm' => MARKET_TRPARM, 'trplnd' => MARKET_TRPLND, 'trpfly' => MARKET_TRPFLY, 'trpsea' => MARKET_TRPSEA, 'food' => MARKET_FOOD);
$defcost = array('trparm' => PVTM_TRPARM, 'trplnd' => PVTM_TRPLND, 'trpfly' => PVTM_TRPFLY, 'trpsea' => PVTM_TRPSEA, 'food' => PVTM_FOOD);

// Determines the current selling price of a unit
function getCost ($emp, $type, $default)
{
	global $db, $stores;
	$q = $db->prepare('SELECT MIN(k_price) AS k_price FROM '. MARKET_TABLE .' WHERE k_type = ? AND e_id != ? AND k_time <= ?');
	db_bindIntValue($q, 1, $stores[$type]);
	db_bindIntValue($q, 2, $emp->e_id);
	db_bindIntValue($q, 3, CUR_TIME);
	$q->execute() or warning('Failed to determine market prices', 0);
	$row = $q->fetch();
	$cost = $row['k_price'];
	if ($cost == 0)
		$cost = $default;
	return $cost;
}

// Gets all items on sale and returns how many more of that type can be sold
// Also builds a list so we can display them in table format at our own convenience
function calcBasket ($emp, $type, &$minsell, &$maxsell, &$sales)
{
	global $db, $stores;
	$q = $db->prepare('SELECT k_id,k_type,e_id,k_amt,k_price,k_time FROM '. MARKET_TABLE .' WHERE k_type = ? AND e_id = ?');
	db_bindIntValue($q, 1, $stores[$type]);
	db_bindIntValue($q, 2, $emp->e_id);
	$q->execute() or warning('Failed to fetch market items', 0);
	$items = $q->fetchAll();
	$onsale = 0;
	foreach ($items as $item)
	{
		$sales[$item['k_id']] = $item;
		$onsale += $item['k_amt'];
	}
	if ($stores[$type] == MARKET_FOOD)
	{
		$minsell[$type] = max(round(($emp->getData('e_'. $type) + $onsale) * PUBMKT_MINFOOD / 100), 0);
		$maxsell[$type] = max(round(($emp->getData('e_'. $type) + $onsale) * PUBMKT_MAXFOOD / 100) - $onsale, 0);
	}
	else
	{
		$minsell[$type] = max(round(($emp->getData('e_'. $type) + $onsale) * PUBMKT_MINSELL / 100), 0);
		$maxsell[$type] = max(round(($emp->getData('e_'. $type) + $onsale) * PUBMKT_MAXSELL / 100) - $onsale, 0);
	}
}

$sales = array();
$minsell = array();
$maxsell = array();
foreach ($types as $type)
	calcBasket($emp1, $type, $minsell, $maxsell, $sales);

if ($action == 'sell') do
{
	$q = $db->prepare('INSERT INTO '. MARKET_TABLE .' (k_type,e_id,k_amt,k_price,k_time) VALUES (?,?,?,?,?)');
	db_bindIntValue($q, 2, $emp1->e_id);
	db_bindIntValue($q, 5, CUR_TIME);

	foreach ($types as $type)
	{
		$minprice = $defcost[$type] * 0.2;
		$maxprice = $defcost[$type] * 2.5;
		$amount = fixInputNum(getFormVar('sell_'. $type));
		$price = fixInputNum(getFormVar('price_'. $type));
		if ($amount == 0)
			continue;
		if ($amount > $maxsell[$type])
		{
			notice(lang('PUBMARKETSELL_TOO_MANY_UNITS', number($maxsell[$type]), $emp1->era->getData($type)));
			continue;
		}
		if ($amount < $minsell[$type])
		{
			notice(lang('PUBMARKETSELL_TOO_FEW_UNITS', number($minsell[$type]), $emp1->era->getData($type)));
			continue;
		}
		if ($price < $minprice)
		{
			notice(lang('PUBMARKETSELL_PRICE_TOO_LOW', $emp1->era->getData($type), money($minprice)));
			continue;
		}
		if ($price > $maxprice)
		{
			notice(lang('PUBMARKETSELL_PROCE_TOO_HIGH', $emp1->era->getData($type), money($maxprice)));
			continue;
		}
		$emp1->subData('e_'. $type, $amount);
		$maxsell[$type] -= $amount;

		db_bindIntValue($q, 1, $stores[$type]);
		db_bindIntValue($q, 3, $amount);
		db_bindIntValue($q, 4, $price);
		$q->execute() or warning('Failed to add items to market', 0);
		$id = $db->lastInsertId();
		// and add it to $sales so we can display it in the list
		$item = array('k_id' => $id, 'k_type' => $stores[$type], 'e_id' => $emp1->e_id, 'k_amt' => $amount, 'k_price' => $price, 'k_time' => CUR_TIME + 3600 * PUBMKT_START);
		$sales[$id] = $item;
		logevent(varlist(array('type', 'amount', 'price'), get_defined_vars()));
	}
} while (0);
if ($action == 'remove') do
{
	if (PUBMKT_MINTIME < 0)
		break;	// not allowed to manually remove units

	$id = fixInputNum(getFormVar('remove_id'));
	// first, try to lock it
	$q = $db->prepare('UPDATE '. MARKET_TABLE .' SET lock_id = ? WHERE k_id = ? AND e_id = ? AND lock_id = 0');
	db_bindIntValue($q, 1, $user1->u_id);
	db_bindIntValue($q, 2, $id);
	db_bindIntValue($q, 3, $emp1->e_id);
	$q->execute() or warning('Failed to lock items to remove from market', 0);
	if ($q->rowCount() == 0)
	{
		notice(lang('PUBMARKETSELL_CANNOT_REMOVE'));
		break;
	}
	// now that it's locked, load it
	$q = $db->prepare('SELECT * FROM '. MARKET_TABLE .' WHERE k_id = ?');
	db_bindIntValue($q, 1, $id);
	$q->execute() or warning('Failed to fetch locked items to remove from market', 0);
	$item = $q->fetch();	// from the above check, the row is guaranteed to exist

	if ($item['k_time'] <= CUR_TIME - 3600 * PUBMKT_MINTIME)
	{
		// remove it from the market, so as to not hold up other buyers
		$q = $db->prepare('DELETE FROM '. MARKET_TABLE .' WHERE k_id = ?');
		$q->execute(array($id)) or warning('Failed to delete items from market', 0);

		$type = $types[$item['k_type']];
		$amount = $item['k_amt'];
		$returned = floor($amount * 0.8);

		$emp1->addData('e_'. $type, $returned);
		notice(lang('PUBMARKETSELL_REMOVE_COMPLETE', number($item['k_amt']), $emp1->era->getData($type), number($returned)));
		logevent(varlist(array('type', 'amount', 'returned'), get_defined_vars()));
	}
	else
	{
		$q = $db->prepare('UPDATE '. MARKET_TABLE .' SET lock_id = 0 WHERE k_id = ? AND lock_id = ?');
		db_bindIntValue($q, 1, $id);
		db_bindIntValue($q, 2, $user1->u_id);
		$q->execute() or warning('Failed to unlock items to remove from market', 0);

		notice(lang('PUBMARKETSELL_REMOVE_TOO_SOON'));
	}
} while (0);
notices();

$costs = array();
foreach ($types as $type)
	$costs[$type] = getCost($emp1, $type, $defcost[$type]);
?>
<?=lang('PUBMARKETSELL_SALES_HEADER')?>
<table class="inputtable">
<tr><th class="al"><?=lang('COLUMN_UNIT')?></th>
    <th class="ar"><?=lang('COLUMN_QUANTITY')?></th>
    <th class="ar"><?=lang('COLUMN_PRICE')?></th>
    <th class="ar"><?=lang('COLUMN_STATUS')?></th></tr>
<?
foreach ($sales as $item)
{
	$type = $types[$item['k_type']];
?>
<tr><td><?=lang($emp1->era->getData($type))?></td>
    <td class="ar"><?=number($item['k_amt'])?></td>
    <td class="ar"><?=money($item['k_price'])?></td>
    <td class="ar"><?
	$itemtime = CUR_TIME - $item['k_time'];
	if ($itemtime >= 0)
	{
		echo lang('PUBMARKETSELL_SALES_ONSALE', round($itemtime / 3600, 1));
		if ((PUBMKT_MINTIME >= 0) && ($itemtime >= 3600 * PUBMKT_MINTIME))
			echo ' <a href="?location=pubmarketsell&amp;action=remove&amp;remove_id='. $item['k_id'] .'">'. lang('PUBMARKETSELL_SALES_REMOVE') .'</a>';
	}
	else	echo lang('PUBMARKETSELL_SALES_INTRANSIT', round(-$itemtime / 3600, 1));
?></td></tr>
<?
}
?>
    <tr><td colspan="4"><hr /></td></tr>
</table>
<?=lang('PUBMARKETSELL_SALES_DELAY', PUBMKT_START)?><br />
<form method="post" action="?location=pubmarketsell">
<table class="inputtable">
<tr><td colspan="2"><a href="?location=pubmarketbuy"><?=lang('PUBMARKET_LINK_BUY')?></a></td>
    <td>&nbsp;</td>
    <td colspan="2" class="ar"><a href="?location=pubmarketsell"><?=lang('PUBMARKET_LINK_SELL')?></a></td></tr>
<tr><th class="al"><?=lang('COLUMN_UNIT')?></th>
    <th class="ar"><?=lang('COLUMN_OWNED')?></th>
    <th class="ar"><?=lang('COLUMN_PRICE')?></th>
    <th class="ar"><?=lang('COLUMN_CANSELL')?></th>
    <th class="ar"><?=lang('COLUMN_SELL')?></th></tr>
<?
foreach ($types as $type)
{
?>
<tr><td><?=lang($emp1->era->getData($type))?></td>
    <td class="ar"><?=number($emp1->getData('e_'. $type))?></td>
    <td class="ar"><input type="text" name="price_<?=$type?>" value="<?=money($costs[$type])?>" size="5" /></td>
    <td class="ar"><?=number($maxsell[$type])?></td>
    <td class="ar"><input type="text" name="sell_<?=$type?>" value="<?=$minsell[$type]?>" size="8" /></td></tr>
<?
}
?>
<tr><td colspan="5" class="ac"><input type="hidden" name="action" value="sell" /><input type="submit" value="<?=lang('PUBMARKETSELL_SUBMIT')?>" /></td></tr>
</table>
</form>
<?
require(PROM_BASEDIR .'includes/footer.php');
?>
