<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/turns.php $
 * $Id: turns.php 1360 2009-01-01 21:27:41Z Quietust $
 */

error_reporting(E_ALL | E_STRICT);
date_default_timezone_set('UTC');
ignore_user_abort(1);
set_time_limit(0);

if (isset($_SERVER['REQUEST_URI']) || defined('IN_GAME'))
	die("Access denied");

define('IN_TURNS', TRUE);
define('PROM_BASEDIR', dirname(__FILE__) . '/');

// Don't allow accessing anything while setup.php is being run
if (file_exists(PROM_BASEDIR .'setup.php'))
	die("Access denied");

require(PROM_BASEDIR .'config.php');
require(PROM_BASEDIR .'includes/constants.php');
require(PROM_BASEDIR .'includes/language.php');
require(PROM_BASEDIR .'includes/misc.php');
require(PROM_BASEDIR .'includes/logging.php');
require(PROM_BASEDIR .'includes/news.php');
require(PROM_BASEDIR .'includes/database.php');
require(PROM_BASEDIR .'classes/prom_vars.php');
require(PROM_BASEDIR .'classes/prom_user.php');
require(PROM_BASEDIR .'classes/prom_empire.php');
require(PROM_BASEDIR .'classes/prom_clan.php');

// these should be sampled when the script first starts (skipping DB auth time)
// but the rest should be dynamic so we can see how long it takes each part to run
$hour = gmdate('H', CUR_TIME);
$min = gmdate('i', CUR_TIME);

function statecho ($message)
{
	global $hour, $min;
	$time = explode(' ', microtime());
	echo sprintf('%s.%06d - [%02d:%02d] - ', date('Y/m/d H:i:s', $time[1]), $time[0] * 1000000, $hour, $min) . $message ."\n";
}

function lock_all ()
{
	global $world, $db;
	statecho('Locking world variables');
	// Use standard locking on world variables; this way, if the turns script gets queued up
	// multiple times, they won't all rush through at the same time after the first is done
	// Delay 50-100ms before each lock retry
	while (1)
	{
		$worldlock = $world->lock(LOCK_TURNS);
		if ($worldlock)
			break;
		usleep(mt_rand(50000, 100000));
	}
	statecho('Locking users');
	while (1)
	{
		// MySQL supports the syntax "INSERT IGNORE", which we can use to lock all unlocked entities in bulk
		if (DB_TYPE == 'mysql')
			sqlquery($db, 'INSERT IGNORE INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) SELECT u_id,?,? FROM '. USER_TABLE, array(ENT_USER, LOCK_TURNS));
		else	// However, other databases do not, so for those we need to add the locks one at a time
		{
			$q = sqlquery('SELECT u_id FROM '. USER_TABLE .' WHERE u_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_USER, LOCK_TURNS));
			$ids = $q->fetchAll();
			$q = $db->prepare('INSERT INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) VALUES (?,?,?)');
			db_bindIntValue($q, 2, ENT_USER);
			db_bindIntValue($q, 3, LOCK_TURNS);
			foreach ($ids as $id)
			{
				db_bindIntValue($q, 1, $id);
				$q->execute();
			}
		}
		if (sqleval($db, 'SELECT COUNT(*) FROM '. USER_TABLE .' WHERE u_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_USER, LOCK_TURNS)) == 0)
			break;
		usleep(mt_rand(50000, 100000));
	}
	statecho('Locking empires');
	while (1)
	{
		// MySQL supports the syntax "INSERT IGNORE", which we can use to lock all unlocked entities in bulk
		if (DB_TYPE == 'mysql')
			sqlquery($db, 'INSERT IGNORE INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) SELECT e_id,?,? FROM '. EMPIRE_TABLE, array(ENT_EMPIRE, LOCK_TURNS));
		else	// However, other databases do not, so for those we need to add the locks one at a time
		{
			$q = sqlquery('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE e_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_EMPIRE, LOCK_TURNS));
			$ids = $q->fetchAll();
			$q = $db->prepare('INSERT INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) VALUES (?,?,?)');
			db_bindIntValue($q, 2, ENT_EMPIRE);
			db_bindIntValue($q, 3, LOCK_TURNS);
			foreach ($ids as $id)
			{
				db_bindIntValue($q, 1, $id);
				$q->execute();
			}
		}
		if (sqleval($db, 'SELECT COUNT(*) FROM '. EMPIRE_TABLE .' WHERE e_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_EMPIRE, LOCK_TURNS)) == 0)
			break;
		usleep(mt_rand(50000, 100000));
	}
	if (CLAN_ENABLE)
	{
		statecho('Locking clans');
		while (1)
		{
			// MySQL supports the syntax "INSERT IGNORE", which we can use to lock all unlocked entities in bulk
			if (DB_TYPE == 'mysql')
				sqlquery($db, 'INSERT IGNORE INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) SELECT c_id,?,? FROM '. CLAN_TABLE, array(ENT_CLAN, LOCK_TURNS));
			else	// However, other databases do not, so for those we need to add the locks one at a time
			{
				$q = sqlquery('SELECT c_id FROM '. CLAN_TABLE .' WHERE c_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_CLAN, LOCK_TURNS));
				$ids = $q->fetchAll();
				$q = $db->prepare('INSERT INTO '. LOCK_TABLE .' (lock_id,lock_type,lock_owner) VALUES (?,?,?)');
				db_bindIntValue($q, 2, ENT_CLAN);
				db_bindIntValue($q, 3, LOCK_TURNS);
				foreach ($ids as $id)
				{
					db_bindIntValue($q, 1, $id);
					$q->execute();
				}
			}
			if (sqleval($db, 'SELECT COUNT(*) FROM '. CLAN_TABLE .' WHERE c_id NOT IN (SELECT lock_id FROM '. LOCK_TABLE .' WHERE lock_type = ? AND lock_owner = ?)', array(ENT_CLAN, LOCK_TURNS)) == 0)
				break;
			usleep(mt_rand(50000, 100000));
		}
	}
	statecho('Lock complete');
}

function unlock_all ()
{
	global $world, $db;
	// save world variables first, but keep them locked
	statecho('Saving world variables');
	$world->save(FALSE);
	statecho('Unlocking entities');
	// then remove all locks at once
	sqlquery($db, 'DELETE FROM '. LOCK_TABLE .' WHERE lock_owner = ?', array(LOCK_TURNS));
	statecho('Unlock complete');
}

$starttime = microtime(TRUE);
statecho('Beginning turn run');

$db = db_open(DB_TYPE, DB_SOCK, DB_HOST, DB_PORT, DB_USER, DB_PASS, DB_NAME);
if (!$db)
{
	statecho('Database is offline - aborting');
	echo "\n";
	exit;
}

$world = new prom_vars();
$world->load();
if (!isset($world->round_time_begin))
	die("Game setup is not yet complete");
lock_all();

// if round hasn't started, update idle times so empires don't expire
if (CUR_TIME < $world->round_time_begin)
{
	statecho('Round has not yet started - updating idle times');
	sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_idle = ?', array(CUR_TIME));
	unlock_all();
	echo "\n";
	exit;
}

// if round has already ended, just exit
if (CUR_TIME >= $world->round_time_end)
{
	statecho('Round has ended - nothing to do');
	unlock_all();
	echo "\n";
	exit;
}

statecho('Starting world variable adjustment');
$q = $db->query('SELECT * FROM '. VAR_ADJUST_TABLE);
$adjusts = $q->fetchAll();
foreach ($adjusts as $adjust)
	$world->setData($adjust['v_name'], $world->getData($adjust['v_name']) + $adjust['v_offset']);
$db->query('DELETE FROM '. VAR_ADJUST_TABLE);
statecho('Finished world variable adjustment');

if ($min == TURNS_OFFSET)
{
	if ($hour == 12)
	{
		statecho('Starting daily events');

		statecho('Starting lottery');
		$numplayers = sqleval($db, 'SELECT COUNT(*) FROM '. EMPIRE_TABLE);
		$lotterynum = mt_rand(1, LOTTERY_MAXTICKETS * $numplayers);

		$jackpot = $world->lotto_current_jackpot;
		$lastjackpot = $world->lotto_yesterday_jackpot;

		// if the minimum jackpot gets increased, apply it now
		if ($lastjackpot > $jackpot)
			$lastjackpot = LOTTERY_JACKPOT;

		$world->lotto_last_picked = $lotterynum;
		$world->lotto_yesterday_jackpot = $jackpot;
		$world->lotto_jackpot_increase = $jackpot - $lastjackpot;

		$win = sqleval($db, 'SELECT e_id FROM '. LOTTERY_TABLE .' WHERE e_id > 0 AND l_ticket = ?', array($lotterynum));
		if ($win)
		{
			$emp_b = new prom_empire($win);
			$emp_b->loadPartial();
			addEmpireNews(EMPNEWS_ATTACH_LOTTERY, NULL, $emp_b, $jackpot);
			statecho('Lottery results: ticket '. prenum($lotterynum) .' held by '. $emp_b .', '. money($jackpot) .' won.');
			$world->lotto_current_jackpot = LOTTERY_JACKPOT;
			$emp_b = NULL;
		}
		else
		{
			statecho('Lottery results: ticket '. prenum($lotterynum) .' not held, no winner.');
			$win = 0;
		}
		$world->lotto_last_winner = $win;
		sqlquery($db, 'DELETE FROM '. LOTTERY_TABLE);
		statecho('Finished lottery');

		// Once per day, give points based on networth
		if (SCORE_ENABLE)
			sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_score = e_score + SQRT(e_networth / 1250000)');
		statecho('Finished daily events');
	}

	statecho('Starting hourly events');
	// Various counters
	if (AID_ENABLE)
		sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_aidcred = e_aidcred + 1 WHERE e_aidcred < ? AND u_id != 0', array(AID_MAXCREDITS));
	sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_vacation = e_vacation + 1 WHERE e_vacation > 0 AND u_id != 0');

	if (MAX_ATTACKS > 0)
	{
		sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks + 1 WHERE e_attacks < 0 AND u_id != 0');
		sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks + 1 WHERE e_attacks < 0 AND u_id != 0');
	}

	// the vacation check here MUST match the behavior in prom_empire::is_vacation_done()
	if (CUR_TIME >= $world->round_time_closing)
		sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_vacation = 0, e_idle = ? WHERE e_vacation >= ? AND u_id != 0', array(CUR_TIME, VACATION_START + VACATION_LIMIT + 1));

	// clean up expired effects
	sqlquery($db, 'DELETE FROM '. EMPIRE_EFFECT_TABLE .' WHERE ef_name LIKE ? AND ef_value <= ?', array(EMPIRE_EFFECT_TIME .'%', CUR_TIME));
	sqlquery($db, 'DELETE FROM '. EMPIRE_EFFECT_TABLE .' WHERE ef_name LIKE ? AND ef_value <= 0', array(EMPIRE_EFFECT_TURN .'%'));

	statecho('Finished hourly events');
}
statecho('Starting normal events');

// Give turns, overflowing into stored turns if necessary
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_turns = e_turns + ? WHERE e_vacation = 0 AND e_flags & ? = 0 AND u_id != 0', array(TURNS_COUNT, EFLAG_DISABLE));
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_turns = e_turns + 1, e_storedturns = e_storedturns - 1 WHERE e_vacation = 0 AND e_flags & ? = 0 AND e_storedturns > 0 AND u_id != 0', array(EFLAG_DISABLE));
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_storedturns = e_storedturns + e_turns - ?, e_turns = ? WHERE e_turns > ? AND u_id != 0', array(TURNS_MAXIMUM, TURNS_MAXIMUM, TURNS_MAXIMUM));
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_storedturns = ? WHERE e_storedturns > ? AND u_id != 0', array(TURNS_STORED, TURNS_STORED));

// Reduce maximum private market sell percentage (by 1% base, up to 2% if the player has nothing but bldcash)
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktperarm = e_mktperarm - LEAST(e_mktperarm, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktperlnd = e_mktperlnd - LEAST(e_mktperlnd, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktperfly = e_mktperfly - LEAST(e_mktperfly, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktpersea = e_mktpersea - LEAST(e_mktpersea, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0');

// Refill private market based on bldcost (except for food, which uses bldfood)
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktarm = e_mktarm + 8 * (e_land + e_bldcost) WHERE e_mktarm < 250 * (e_land + 2 * e_bldcost) AND u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktlnd = e_mktlnd + 5 * (e_land + e_bldcost) WHERE e_mktlnd < 200 * (e_land + 2 * e_bldcost) AND u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktfly = e_mktfly + 3 * (e_land + e_bldcost) WHERE e_mktfly < 180 * (e_land + 2 * e_bldcost) AND u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktsea = e_mktsea + 2 * (e_land + e_bldcost) WHERE e_mktsea < 150 * (e_land + 2 * e_bldcost) AND u_id != 0');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_mktfood = e_mktfood + 50 * (e_land + e_bldfood) WHERE e_mktfood < 2000 * (e_land + 2 * e_bldfood) AND u_id != 0');

// Various other counters
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_msgcred = e_msgcred + 1 WHERE e_msgcred < ? AND u_id != 0', array(MESSAGES_MAXCREDITS));
if (MAX_ATTACKS > 0)
	sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks - 1 WHERE e_attacks > 0 AND u_id != 0');
if (CLAN_ENABLE)
	sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_sharing = e_sharing - 1 WHERE e_sharing > 0 AND u_id != 0');

statecho('Finished normal events');

if (PUBMKT_MAXTIME >= 0)
{
	statecho('Starting market cleanup');

	$q = sqlquery($db, 'SELECT * FROM '. MARKET_TABLE .' WHERE k_time <= ?', array(CUR_TIME - 3600 * PUBMKT_MAXTIME));
	$markets = $q->fetchAll();
	$defcost = array(MARKET_TRPARM => PVTM_TRPARM, MARKET_TRPLND => PVTM_TRPLND, MARKET_TRPFLY => PVTM_TRPFLY, MARKET_TRPSEA => PVTM_TRPSEA, MARKET_FOOD => PVTM_FOOD);
	foreach ($markets as $market)	// send them back
	{
		$emp_b = new prom_empire($market['e_id']);
		$emp_b->loadPartial();
		$basecost = $defcost[$market['k_type']];
		$lost = floor($market['k_amt'] * (min(max($market['k_price'] - $basecost, 0) / $basecost, 0.3) + 0.2));
		addEmpireNews(EMPNEWS_ATTACH_MARKET_RETURN, NULL, $emp_b, $market['k_type'], $market['k_amt'], $market['k_price'], $market['k_amt'] - $lost);
		$emp_b = NULL;
	}
	sqlquery($db, 'DELETE FROM '. MARKET_TABLE .' WHERE k_time <= ?', array(CUR_TIME - 3600 * PUBMKT_MAXTIME));	// and delete them
	statecho('Finished market cleanup');
}

statecho('Starting empire cleanup');
// the protection check here MUST match the behavior in prom_empire::is_protected()
$q = sqlquery($db, 'SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 AND e_flags & ? = 0 AND ('.
	'(e_flags & ? = ? AND e_idle < ?) OR '.
	'(e_flags & ? = ? AND e_idle < ?) OR '.
	'(e_flags & ? = 0 AND e_vacation = 0 AND e_land > 0 AND e_idle < ?) OR '.
	'(e_land = 0 AND (e_flags & ? = ? OR e_idle < ?)) OR '.
	'(e_flags & ? = ? AND (e_turnsused <= ? OR e_idle < ?))'.
		')', array(EFLAG_ADMIN,						// don't ever delete admins
	EFLAG_VALID | EFLAG_NOTIFY, 0, CUR_TIME - 86400 * IDLE_TIMEOUT_NEW,			// not validated or notified
	EFLAG_VALID | EFLAG_NOTIFY, EFLAG_NOTIFY, CUR_TIME - 86400 * IDLE_TIMEOUT_VALIDATE,	// notified but not validated
	EFLAG_DISABLE, CUR_TIME - 86400 * IDLE_TIMEOUT_ABANDON, 				// not disabled or on vacation
	EFLAG_NOTIFY, EFLAG_NOTIFY, CUR_TIME - 86400 * IDLE_TIMEOUT_KILLED,			// dead (immediate if notified)
	EFLAG_DELETE, EFLAG_DELETE, TURNS_PROTECTION, CUR_TIME - 86400 * IDLE_TIMEOUT_DELETE,	// marked for deletion (immediate if still under protection)
		));
$emps = $q->fetchAll();

foreach ($emps as $row)
{
	$emp_a = new prom_empire($row['e_id'], FALSE); // do not require lock
	$emp_a->lock(LOCK_TURNS);
	statecho('Unlinking empire '. $emp_a);
	$reason = '';
	if (CLAN_ENABLE && $emp_a->c_id)
	{	// remove user from clan
		$clan_a = new prom_clan($emp_a->c_id, FALSE); // do not require lock
		$clan_a->lock(LOCK_TURNS);
		if ($clan_a->e_id_leader == $emp_a->e_id)
		{	// transfer ownership if possible
			$reason .= 'clan leader, ';
			if ($clan_a->e_id_asst)
			{
				$clan_a->e_id_leader = $clan_a->e_id_asst;
				$clan_a->e_id_asst = 0;

				$emp_b = new prom_empire($clan_a->e_id_leader);
				$emp_b->loadPartial();
				addEmpireNews(EMPNEWS_CLAN_INHERIT_LEADER, $emp_a, $emp_b, 0);
				addClanNews(CLANNEWS_PERM_ASSISTANT_INHERIT, $clan_a, $emp_b, NULL, $emp_a);
				$reason .= 'to asst '. $emp_b .', ';
				$emp_b = NULL;
			}
			elseif ($found = sqleval($db, 'SELECT e_id FROM '. EMPIRE_TABLE .' WHERE c_id = ? AND e_id != ? AND e_flags & ? = 0 AND e_land > 0 ORDER BY e_networth DESC LIMIT 1', array($clan_a->c_id, $emp_a->e_id, EFLAG_DELETE | EFLAG_DISABLE)))
			{
				$event = CLANNEWS_PERM_MEMBER_INHERIT;
				$clan_a->e_id_leader = $found;
				if ($clan_a->e_id_fa1 == $found)
				{
					$clan_a->e_id_fa1 = 0;
					$event = CLANNEWS_PERM_MINISTER_INHERIT;
				}
				if ($clan_a->e_id_fa2 == $found)
				{
					$clan_a->e_id_fa2 = 0;
					$event = CLANNEWS_PERM_MINISTER_INHERIT;
				}

				$emp_b = new prom_empire($found);
				$emp_b->loadPartial();
				addEmpireNews(EMPNEWS_CLAN_INHERIT_LEADER, $emp_a, $emp_b, 0);
				addClanNews($event, $clan_a, $emp_b, NULL, $emp_a);
				$reason .= 'to member '. $emp_b .', ';
				$emp_b = NULL;
			}
			else
			{
				$clan_a->e_id_leader = 0;
				$reason .= 'abandoned, ';
			}
		}
		elseif ($clan_a->e_id_asst == $emp_a->e_id)
		{
			$clan_a->e_id_asst = 0;
			$reason .= 'clan asst, ';
		}
		elseif ($clan_a->e_id_fa1 == $emp_a->e_id)
		{
			$clan_a->e_id_fa1 = 0;
			$reason .= 'clan fa1, ';
		}
		elseif ($clan_a->e_id_fa2 == $emp_a->e_id)
		{
			$clan_a->e_id_fa2 = 0;
			$reason .= 'clan fa2, ';
		}
		$clan_a->c_members--;
		$clan_a->save();
		$clan_a = NULL;

		$emp_a->c_oldid = $emp_a->c_id;
		$emp_a->c_id = 0;		// remove from clan
	}
	// delete any messages to/from that user
	sqlquery($db, 'UPDATE '. EMPIRE_MESSAGE_TABLE .' SET m_flags = m_flags | ? WHERE e_id_src = ? OR e_id_dst = ?', array(MFLAG_DELETE, $emp_a->e_id, $emp_a->e_id));
	// any of the user's items on the market
	sqlquery($db, 'DELETE FROM '. MARKET_TABLE .' WHERE e_id = ?', array($emp_a->e_id));
	// any lottery tickets
	sqlquery($db, 'DELETE FROM '. LOTTERY_TABLE .' WHERE e_id = ?', array($emp_a->e_id));

	if ($emp_a->e_flags & EFLAG_DELETE)
	{
		if ($emp_a->e_killedby == 0)
		{
			$reason .= 'deleted';
			$emp_a->e_reason = 'Self-deleted';
			$emp_a->e_killedby = $emp_a->e_id;
		}
		else
		{
			$reason .= 'nuked';
			$emp_a->e_reason = 'Deleted by admin: '. $emp_a->e_reason;
		}
	}
	elseif (($emp_a->e_flags & (EFLAG_VALID | EFLAG_NOTIFY)) == 0)
	{
		$reason .= 'ignored';
		$emp_a->e_reason = 'Auto-deleted, never used enough turns to require validation';
		$emp_a->e_killedby = 0;
	}
	elseif (($emp_a->e_flags & (EFLAG_VALID | EFLAG_NOTIFY)) == EFLAG_NOTIFY)
	{
		$reason .= 'unvalidated';
		$emp_a->e_reason = 'Auto-deleted, failed to validate';
		$emp_a->e_killedby = 0;
	}
	elseif ($emp_a->e_land == 0)
	{
		$reason .= 'killed';
		$emp_a->e_reason = 'Killed';
	}
	elseif ((($emp_a->e_flags & EFLAG_DISABLE) == 0) && ($emp_a->e_vacation == 0))
	{
		$reason .= 'abandoned';
		$emp_a->e_reason = 'Auto-deleted, left idle for an excessive duration';
		$emp_a->e_killedby = 0;
	}
	else
	{
		$reason .= 'indeterminate';
		$emp_a->e_reason = 'Unknown deletion reason';
		$emp_a->e_killedby = 0;
	}

	$emp_a->u_oldid = $emp_a->u_id;
	$emp_a->u_id = 0;		// unlink from user
	$emp_a->save();
	statecho('Empire '. $emp_a .' unlinked - '. $reason);
	$emp_a = NULL;
}
statecho('Finished empire cleanup');

if (CLAN_ENABLE)
{
	statecho('Starting clan cleanup');
	// need a dummy record here so we can record the clan of origin
	$emp_a = new prom_news_placeholder();
	$q = sqlquery($db, 'SELECT c_id FROM '. CLAN_TABLE .' WHERE c_members = 0');
	$clans = $q->fetchAll();
	foreach ($clans as $row)
	{	// remove all associations with empty clans and make clan invisible to game
		$clan_a = new prom_clan($row['c_id'], FALSE);
		$clan_a->lock(LOCK_TURNS);
		statecho('Cleaning clan '. $clan_a);
		// can't actually delete the row, since news table entries will still refer to it
		$clan_a->c_members = -1;
		$emp_a->c_id = $clan_a->c_id;	// assign clan ID to dummy record

		$q1 = sqlquery($db, 'SELECT e_id_leader AS e_id,cr_flags,c_id FROM '. CLAN_RELATION_TABLE .' LEFT OUTER JOIN '. CLAN_TABLE .' ON (c_id_2 = c_id) WHERE c_id_1 = ?', array($clan_a->c_id));
		$q2 = sqlquery($db, 'SELECT e_id_leader AS e_id,cr_flags,c_id FROM '. CLAN_RELATION_TABLE .' LEFT OUTER JOIN '. CLAN_TABLE .' ON (c_id_1 = c_id) WHERE c_id_2 = ?', array($clan_a->c_id));
		$rels = array_merge($q1->fetchAll(), $q2->fetchAll());
		foreach ($rels as $rel)
		{
			$emp_b = new prom_empire($rel['e_id']);
			$emp_b->loadPartial();
			$clan_b = new prom_news_placeholder(0, $rel['c_id']);
			if ($rel['cr_flags'] & CRFLAG_ALLY)
			{
				$empevent = EMPNEWS_CLAN_ALLY_GONE;
				$clanevent = CLANNEWS_RECV_ALLY_GONE;
			}
			else
			{
				$empevent = EMPNEWS_CLAN_WAR_GONE;
				$clanevent = CLANNEWS_RECV_WAR_GONE;
			}
			addEmpireNews($empevent, $emp_a, $emp_b, 0);
			addClanNews($clanevent, $clan_b, NULL, $clan_a);
			$clan_b = NULL;
			$emp_b = NULL;
		}
		sqlquery($db, 'DELETE FROM '. CLAN_RELATION_TABLE .' WHERE c_id_1 = ? OR c_id_2 = ?', array($clan_a->c_id, $clan_a->c_id));
		$clan_a->save();
		statecho('Clan '. $clan_a .' cleaned.');
		$clan_a = NULL;
	}
	$emp_a = NULL;
	statecho('Finished clan cleanup');
}

statecho('Starting rank update');
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_rank = 0 WHERE u_id = 0');
// If score tracking is enabled, rank players by their score instead of their networth
if (SCORE_ENABLE)
	$q = $db->query('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 ORDER BY e_score DESC, e_networth DESC');
else	$q = $db->query('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 ORDER BY e_networth DESC');
$emps = $q->fetchAll();
$urank = 0;
foreach ($emps as $emp)
{
	$urank++;
	sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_rank = ? WHERE e_id = ?', array($urank, $emp['e_id']));
}
statecho('Finished rank update');

// after 3 turns updates of being idle (or if they've been unlinked), set as offline
sqlquery($db, 'UPDATE '. EMPIRE_TABLE .' SET e_flags = e_flags & ? WHERE e_idle < ? OR u_id = 0', array(~EFLAG_ONLINE, CUR_TIME - 3 * 60 * TURNS_FREQ));

unlock_all();

$duration = microtime(TRUE) - $starttime;
echo "Total time elapsed: $duration seconds\n";
echo "\n";
?>
